# 9 Feb 11
# 	R script that reads merged climate/conflict data & estimates regression models
#		using full violent event count data/cell
#		use negative binomial distribution
#		http://www.ats.ucla.edu/stat/stata/seminars/count_presentation/count.htm
#	GAM references
#		http://ecology.msu.montana.edu/labdsv/R/labs/lab5/lab5.html
#		http://www.statsoft.com/textbook/generalized-additive-models/
#		http://media.wiley.com/product_data/excerpt/17/04703199/0470319917-1.pdf
#	Re-ran May 2012 to correct partial error in TI6 data
#
#	This file is in support of the publication:
#		OLoughlin, J., F. Witmer, A. Linke, A. Laing, A Gettelman, and J. Dudhia (2012).
#		Climate variability and conflict risk in East Africa 1990-2009.
#		Proceedings of the National Academy of Sciences.
#
###############################################

# set output directory
outDir <- "C://Users/XXX/"

# load model analsys functions
source(paste(outDir, "../SharedClimateModelFunctions.R", sep=""))

# load grid-month data & define regression variables for analysis
source(paste(outDir, "../LoadOneDegGridMonthData.R", sep=""))

## PNAS Table S1.
summVarList <- c(depVar, "UCDP_Onsets", "SPI6", "TI6", controlVars[1:(length(controlVars)-2)], "DMI_lag4", "GrowSeason")
summVars <- data.frame(Vars=summVarList)
summVars$Min <- apply(longData[, summVarList], MARGIN=2, FUN=min)
summVars$Median <- apply(longData[, summVarList], MARGIN=2, FUN=median)
summVars$Mean <- apply(longData[, summVarList], MARGIN=2, FUN=mean)
summVars$Max <- apply(longData[, summVarList], MARGIN=2, FUN=max)
summVars$StdDev <- apply(longData[, summVarList], MARGIN=2, FUN=sd)
write.csv(summVars, file=paste(outDir, "varSumms.csv", sep=""))

#longData[which(longData$Events == 80),]	# rowID = 197.200308 --> Mogadishu, Aug 2003
longData[which(longData$Events == 53),]	# rowID = 96.200009 --> Burundi, Sep 2000
length(longData$Events[longData$Events > 0])/length(longData$Events)*100	# 5.9% non-zero

########### GLM & GAM neg. bin. models ######################

## PNAS Table 1, col. a) glm model, no splines
precipVar <- "SPI6"; tempVar <- "TI6"
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
baseNB <- myGLM(depVar, glmVars, writeFile=TRUE, "Base")	# 97 secs; 55 secs w/ Year fixed effects

## SPI6 extremes variables
longData$SPI6_dry <- (longData$SPI6 <= -1) * 1; sum(longData$SPI6_dry)	# 10855
longData$SPI6_wet <- (longData$SPI6 >= 1) * 1; sum(longData$SPI6_wet)	# 11014
#longData$SPI6_bin <- (longData$SPI6_dry | longData$SPI6_wet) * 1; sum(longData$SPI6_bin)	# 21869

## PNAS Table 1, col. b) SPI6 dry
precipVar <- "SPI6_dry"; tempVar <- "TI6"
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
SPI6_dryNB <- myGLM(depVar, glmVars, writeFile=TRUE, "SPI6_dry")	# 46 secs [execution times 1/2 w/ R 2.15.0]

## PNAS Table 1, col. c) SPI6 wet
precipVar <- "SPI6_wet"; tempVar <- "TI6"
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
SPI6_wetNB <- myGLM(depVar, glmVars, writeFile=TRUE, "SPI6_wet")	# 47 secs

## TI6 extremes variables
longData$TI6_hot <- (longData$TI6 >= 1) * 1; sum(longData$TI6_hot)	# 22556 -> 31882 -> 32196
longData$TI6_cold <- (longData$TI6 <= -1) * 1; sum(longData$TI6_cold)	# 2476 -> 1611 -> 1588

## PNAS Table 1, col. d) TI6_hot
precipVar <- "SPI6"; tempVar <- "TI6_hot"
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
TI6_hotNB <- myGLM(depVar, glmVars, writeFile=TRUE, "TI6_hot")	# 46 secs

## PNAS Table 1, col. e) TI6_cold
precipVar <- "SPI6"; tempVar <- "TI6_cold"
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
TI6_coldNB <- myGLM(depVar, glmVars, writeFile=TRUE, "TI6_cold")	# 46 secs

## Table S5 col. g) 'very hot' binary (TI6 >= 2)
longData$TI6_hot2 <- (longData$TI6 >= 2) * 1; sum(longData$TI6_hot2)	# 5756
precipVar <- "SPI6"; tempVar <- "TI6_hot2"
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
TI6_hot2glm <- myGLM(depVar, glmVars, writeFile=TRUE, "TI6_hot2")	# 46 secs
exp(coef(TI6_hot2glm))

## PNAS Table 1, col. f) base GAM model
depVar <- "Events"
precipVar <- "SPI6"; tempVar <- "TI6"
#baseSmooth <- "cr"	 # test cubic reg., but results are not much different
#baseSmooth <- "ts"	 # test tp w/ shrinkage, results are almost identical [deleted]
gamVars <- c(paste(depVar, "SpLag_lag1", sep=""), paste("s(SPI6, bs=\"",baseSmooth,"\")", sep=""), paste("s(TI6, bs=\"",baseSmooth,"\")", sep=""), controlVars)
thetaNB <- baseNB$theta		# grab theta from glm.nb(); thetaNB <- 0.1206613; w/ Year FE thetaNB <- 0.1287402
# PNAS Fig. 1.
gamBaseCntryFE <- myGAM(depVar, gamVars, thetaNB, writeFile=TRUE, "Base")	# 116 secs
#gam.check(gamBaseCntryFE)
save(gamBaseCntryFE, file=paste(outDir, depVar, "GamBaseCntryFE", baseSmooth, ".Rdata", sep=""), compress=TRUE) # 9 MB
#load(file=paste(outDir, depVar, "gamBaseCntryFE", baseSmooth, ".Rdata", sep=""))


## PNAS Table 2. fit with new data (variable means) & check effect of 1 unit increase for SPI6 & TI6 at different steps
glmVars <- c(paste(depVar, "SpLag_lag1", sep=""), precipVar, tempVar, controlVars)
summary(longData[, glmVars[1:(length(glmVars)-1)]])	# use means as default values (except for CptlDV); Country & Year don't matter
testData <- data.frame(EventsSpLag_lag1=0.1769, SPI6=0.0686, TI6=0.710, Ethnologue=0.0741, BrdrDistLnKm=4.291, CptlDV=0, GPWpopLn=12.331, IMR_lag12=95.27, PR_lag12=5.492, ElectPres=0.059, PctGrassland=37.302, DistToRdLn=2.431, CropPIpct=3.257, VCI_lag6=48.46, Country="Burundi", Year = 1)

## calculate confidence interval at a given set of values
calcConfInt <- function(gamObj, toVals, splineNum) {	# gamObj <- gamBaseCntryFE; splineNum <- 1; toVals <- c(-4:4)
	## grab the following bits from plot.gam() and mgcv:::plot.mgcv.smooth()
	#		similar to http://www.mail-archive.com/r-help@r-project.org/msg171214.html
	#http://stackoverflow.com/questions/7195628/in-r-how-to-find-out-which-method-is-dispatched-for-a-particular-function-call
	#is(gamObj$smooth[[1]])
	xSmooth <- gamObj$smooth[[splineNum]]
	smoothFitCI <- data.frame(x=toVals)
	dat <- data.frame(x = smoothFitCI$x)
	names(dat) <- xSmooth$term
	lpm <- PredictMat(xSmooth, dat)	# could also use predict() w/ type="lpmatrix" here?
	first <- xSmooth$first.para
	last <- xSmooth$last.para
	coeffs <- gamObj$coefficients[first:last]
	smoothFitCI$fits <- lpm %*% coeffs
	smoothFitCI$stderrs <- sqrt(rowSums((lpm %*% gamObj$Vp[first:last, first:last]) * lpm))
	smoothFitCI$loCI <- smoothFitCI$fits - 2 * smoothFitCI$stderrs
	smoothFitCI$hiCI <- smoothFitCI$fits + 2 * smoothFitCI$stderrs
	smoothFitCI$sig <- sign(smoothFitCI$loCI) == sign(smoothFitCI$hiCI)
	smoothFitCI
}

## calculate coefficient difference between lists of from & to Vals
#	also calculate the incidence rate ratio (IRR) or relative risk ratio
calcUnitCoef <- function(gamObj, coefDF, varName, testData) {	# gamObj <- gamBaseCntryFE; varName <- "SPI6"
	for(i in 1:nrow(coefDF)){
		testData[[varName]] <- coefDF$fromVal[i]
		fromLogRisk <- predict(gamObj, newdata = testData)
		testData[[varName]] <- coefDF$toVal[i]
		toLogRisk <- predict(gamObj, newdata = testData)
		coefDF$diffCoef[i] <- toLogRisk - fromLogRisk
		coefDF$IRR[i] <- exp(coefDF$diffCoef[i])
	}
	coefDF
}
spi6coef <- data.frame(fromVal=rep(0, 8), toVal=c(1:4, -1:-4), diffCoef=NA, IRR=NA)
spi6coef <- calcUnitCoef(gamBaseCntryFE, spi6coef, "SPI6", testData)
gamCI <- calcConfInt(gamBaseCntryFE, spi6coef$toVal, 1)
spi6coef$sig <- gamCI$sig
write.csv(spi6coef, file=paste(outDir, "gamBaseCntryFE_SPI6coef", baseSmooth, ".csv", sep=""), row.names=FALSE)

TI6coef <- data.frame(fromVal=rep(0, 8), toVal=c(1:4, -1:-4), diffCoef=NA, IRR=NA)
TI6coef <- calcUnitCoef(gamBaseCntryFE, TI6coef, "TI6", testData)
gamCI <- calcConfInt(gamBaseCntryFE, spi6coef$toVal, 2)
TI6coef$sig <- gamCI$sig
write.csv(TI6coef, file=paste(outDir, "gamBaseCntryFE_TI6coef", baseSmooth, ".csv", sep=""), row.names=FALSE)

## retrieve coeffs for a given SPI6 value
#	a bit of a hack to get around type="terms" error (Simon Wood has since fixed this problem)
retrieveSplineCoef <- function(gamObj, varName, varVal, testData) {
	testData[[varName]] <- varVal
	lpm <- predict(gamBaseCntryFE, newdata = testData, type="lpmatrix")	# linear predictor matrix
	varRange <- grep(varName, colnames(lpm))
	sum(lpm[varRange] * coef(gamBaseCntryFE)[varRange])
}
# these values reported in the text
retrieveSplineCoef(gamBaseCntryFE, "SPI6", 2, testData)			# -0.302
exp(retrieveSplineCoef(gamBaseCntryFE, "SPI6", 2, testData))	# 0.739


### PNAS Fig. 2: AUC predictive power vs. z-score the AUC while deleting each variable
#	Code adapted from Ward MD, Greenhill BD, Bakke KM (2010) The perils of policy by p-value: Predicting civil conflicts. J Peace Res 47:363-375.
# remove FE vars from gamVars list
nb_AUCs <- calcGAMPredPower(depVar, gamVars, thetaNB)	# 19 minutes
save(nb_AUCs, file=paste(outDir, "gamBaseCntryFE", baseSmooth, "_AUCs.Rdata", sep=""), compress=TRUE)
#load(file=paste(outDir, "gamBaseCntryFE", baseSmooth, "_AUCs.Rdata", sep=""))
baseNB_AUC <- calcAUC(gamBaseCntryFE)
allPredPower <- baseNB_AUC - nb_AUCs	# calculate predictive power for all variables
nb_Ppower <- allPredPower[1:(length(allPredPower)-2)]	# remove FE vars
# now extract the z-scores for all the terms in the main model:
# omit Intercept & Country/Year-level FE z-values
# use clustered std. errors
clErrs <- clustErrs(gamBaseCntryFE, longData$Grid)
baseNB_Zs <- clErrs[2:(length(gamVars)-3), 3]		# remove FE vars
baseNB_Zs <- abs(baseNB_Zs)

displayNames<-c("Events space-time lag", "s(SPI6)", "s(TI6)", "Ethnic leadership", "Distance to border (ln)", "Capital city", "Population (ln)", "IMR (lag)", "Political rights (lag)", "Pres. elect.", "Grassland (pct)", "Dist to road (ln)", "CropPI(pct change)", "VegCI")#, "Country", "Year")
# now make the predictive power plot:
filePrefix <- paste("gamBaseCntryFE", baseSmooth, "_PredPower", sep="")
pdf(file=paste(outDir, filePrefix, ".pdf", sep=""), width=7, height=5)
	#par(mar=c(5,6,4,2),mgp=c(3,1,0),las=1)	# Hmmmm, margin fiddling?
	# range(baseNB_Zs); range(nb_Ppower)
	minY <- -0.005
	maxY <- 0.015	#0.017
	maxX <- 15		#80
	fsize <- 1.1
	splineI <- c(2,3)
	plot(baseNB_Zs, nb_Ppower[-splineI], type="n",bty="n",xlim=c(0,maxX),ylim=c(minY,maxY),ylab="Change in predictive power (AUC)",xlab="Grid-clustered statistical significance (absolute z value)", main="Predictive power for variables in GAM neg. bin. model", cex.lab=fsize, cex.axis=fsize)
	segments(x0=1,x1=max(baseNB_Zs),y0=0,y1=0,col="red", lwd=1)	# red line at y=0
	# add a line showing the regression line estimated intercept=0:
	model1<-lm(nb_Ppower[-splineI]~0+baseNB_Zs)
	segments(x0=min(baseNB_Zs),y0=min(baseNB_Zs)*model1$coefficients,x1=max(baseNB_Zs),y1=max(baseNB_Zs)*model1$coefficients, lwd=2, col="grey")
	# add the points and spline lines
	points(baseNB_Zs,nb_Ppower[-splineI],pch=19)
	segments(x0=1,x1=3,y0=nb_Ppower[splineI[1]],y1=nb_Ppower[splineI[1]],col="black", lwd=1, lty=2)
	segments(x0=1.5,x1=3.5,y0=nb_Ppower[splineI[2]],y1=nb_Ppower[splineI[2]],col="black", lwd=1, lty=2)
	
	# add the text:
	h.adj<-c(-4,0.0,0.2,0.1,0.1,  0.0,0.1,0.0,-2.8,0.2,  0.0,-1.2)
	v.adj<-c(0.0007,-0.0015,0.000,0.0005,0.0008,  -0.002,0.0004,-0.0005,0.0005,0.0002,  -0.0028,-0.0008)
	text(baseNB_Zs+h.adj,nb_Ppower[-splineI]+v.adj,displayNames[-splineI],adj=0, cex=0.8)
	
	text(-0.4,nb_Ppower[splineI[1]]+0.0002,displayNames[splineI[1]],adj=0, cex=0.8)
	text(0.5,nb_Ppower[splineI[2]]+0.0007,displayNames[splineI[2]],adj=0, cex=0.8)
dev.off()

