# 12 May 11
# 	Shared functions to analyze climate regression model output
#
#	This file is in support of the publication:
#		OLoughlin, J., F. Witmer, A. Linke, A. Laing, A Gettelman, and J. Dudhia (2012).
#		Climate variability and conflict risk in East Africa 1990-2009.
#		Proceedings of the National Academy of Sciences.
#
###############################################

# comment out the following line after running it once:
install.packages(c('MASS', 'fmsb', 'Hmisc', 'mgcv', 'car', 'sandwich', 'lmtest'))

library('MASS')		# for glm.nb()
library('fmsb')		# for Nagelkerke pseudo-R2
library('Hmisc')	# for somers2()
library('mgcv')		# standard package for gam additive models
library('car')		# for vif()


#*********************************************************************************
# function for accessing polygon slot information, default is "ID"
getPolySlot <- function(poly, sl="ID") {
	sapply(slot(poly, "polygons"), function(x) slot(x, sl)) 
}

##### calculate the area under the curve (AUC) for the given model
# requires model$y to be defined
calcAUC <- function (model, depVar = NULL) {
	fit <- fitted(model)		# same as predict() (for all model types?), but MUCH faster for gam models
#	fit <- predict(model, type="response")	# generally consistent across diff models, fitted() also works fine
	fitBin <- fit
	fitBin[fit > 1] <- 1	# collapse to binary predicted variable
	if (length(depVar) == 0) {
		yBin <- (model$y >= 1) * 1	# convert dependent var to binary
	} else {
		yBin <- (depVar >= 1) * 1
	}
	somers2(fitBin, yBin)[1]
#	somers2(plogis(fitBin), longData$EventsBin)[1]
	# plogis() only needed for predict values with type="link" (though return value is still the same)
}

##### function to calculate clustered standard errors given a fitted model and cluster variable
# clustErrs() source: http://people.su.se/~ma/mcluster.R
#	http://thetarzan.wordpress.com/2011/06/11/clustered-standard-errors-in-r/
#	http://andrewgelman.com/2007/11/clustered_stand/
#	http://people.su.se/~ma/clustering.pdf
#	install.packages(pkgs="lmtest", repos="http://cran.case.edu/")
clustErrs <- function(fm, cluster) {
#clustErrs <- function(dat, fm, cluster) {	fm <- gamBaseCntryFE; cluster <- longData$Grid; fm <- baseNB
#	attach(dat, warn.conflicts = F)
	library(sandwich)
	library(lmtest)
	M <- length(unique(cluster))
	N <- length(cluster)
	K <- fm$rank	# rank might not be set properly in gam objects, but should not matter much since N >> K
	dfc <- (M/(M-1))*((N-1)/(N-K))
	#browser()	# problem w/ CropPIpct_lag12 since no data for 1991 (would require data from 1989)
	#dim(estfun(fm)) # 86832 x 45; length(cluster) # 91656; length(fm$model$Events) # 86832
	uj  <- apply(estfun(fm), 2, function(x) tapply(x, cluster, sum))	# 2 = rows
	#head(uj)
	vcovCL <- dfc*sandwich(fm, meat=crossprod(uj)/N)	# vcov clustered
	head(vcovCL)
	coeftest(fm, vcovCL)
}

#### function to write model coefficients to disk for negbin & ols models
#	requires correct 'longData' object in namespace
writeGLM <- function (modelNB, fname, clustErrors = FALSE, writeTheta=TRUE) {	# modelNB <- cntryGLM_NB
	cfs <- coef(summary(modelNB))#[,-2]
	if (clustErrors == TRUE) {	# assume errors clustered by grid cell
		# TODO: re-write writeGLM() w/ inputDF parameter (or, just a clustVar parameter)
		if (nrow(modelNB$model) == 77748) {	# special case for no Somalia model
			print(paste("Warning: num model obs =", nrow(modelNB$model), "which differs from num grids", length(longData$Grid), " *** Special no Somalia case!"))
			clErrs <- clustErrs(modelNB, longData$Grid[longData$Country != "Somalia"])
		} else if (nrow(modelNB$model) != length(longData$Grid)) {	# a bit of a hack for CropPIpct_lag12 missing 1991 data
			print(paste("Warning: num model obs =", nrow(modelNB$model), "which differs from num grids", length(longData$Grid)))
			clErrs <- clustErrs(modelNB, longData$Grid[!is.na(longData$CropPIpct_lag12)])
		} else {
			clErrs <- clustErrs(modelNB, longData$Grid)
		}
		colnames(clErrs) <- c("Estimate", "Clust. Std. Err.", "Clust. z value", "Clust. Pr(>|z|)")
		cfs <- cbind(cfs, clErrs)
	}
	outData <- rbind(cfs, NA, NA, NA, NA, NA, NA, NA, NA)
	rownames(outData) <- c(rownames(cfs), "theta", "logLik", "AIC", "NagelkerkeR2", "AUC", "DevExpl", "N", "SumEvents")
	if (writeTheta == TRUE)
		outData["theta", 1] <- modelNB$theta
	outData["logLik", 1] <- logLik(modelNB)
	outData["AIC", 1] <- AIC(modelNB)
	outData["NagelkerkeR2", 1] <- NagelkerkeR2(modelNB)$R2
	outData["AUC", 1] <-calcAUC(modelNB)
	outData["DevExpl", 1] <- (modelNB$null.deviance - modelNB$deviance)/modelNB$null.deviance	# from summary.gam()
	outData["N", 1] <- length(modelNB$y)
	outData["SumEvents", 1] <- sum(modelNB$y)
	write.csv(outData, file=paste(outDir, fname, sep=""))
}

#### function to write model coefficients to disk for GAM neg. bin. models
#	can also handle OLS & logit models by setting writeTheta = FALSE
#	default requires correct 'longData' object in namespace
writeGAM <- function (modelNB, fname, numSplineVars = 1, writeTheta=TRUE, clustErrors = FALSE, dataFr=longData) {
#	modelNB <- cntryGAM # gamBaseCntryFE
#	names(modelNB)
#	modelNB$coef
	sumGAM <- summary(modelNB)
#	names(sumGAM)
	cfs <- cbind(sumGAM$p.coeff, sumGAM$se[1:length(sumGAM$p.coeff)], sumGAM$p.t, sumGAM$p.pv)
	colnames(cfs) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
	if (clustErrors == TRUE) {	# assume errors clustered by grid cell
		# TODO: could re-write writeGAM() w/ inputDF parameter (or, just a clustVar parameter)
		if (nrow(modelNB$model) == 77748) {	# special case for no Somalia model
			print(paste("Warning: num model obs =", nrow(modelNB$model), "which differs from num grids", length(longData$Grid), " *** Special no Somalia case!"))
			clErrs <- clustErrs(modelNB, longData$Grid[longData$Country != "Somalia"])
		} else if (nrow(modelNB$model) != length(dataFr$Grid)) {	# a bit of a hack for CropPIpct_lag12 missing 1991 data
			print(paste("Warning: num model obs =", nrow(modelNB$model), "which differs from num grids", length(dataFr$Grid)))
			clErrs <- clustErrs(modelNB, dataFr$Grid[!is.na(dataFr$CropPIpct_lag12)])
		} else {
			clErrs <- clustErrs(modelNB, dataFr$Grid)
		}
		colnames(clErrs) <- c("Estimate", "Clust. Std. Err.", "Clust. z value", "Clust. Pr(>|z|)")
		cfs <- cbind(cfs, clErrs[1:length(sumGAM$p.coeff),])
		#plot(cfs[,3], cfs[,7])
	}
	diagnosticVars <- c("UBRE", "theta", "logLik", "AIC", "NagelkerkeR2", "AUC", "AdjR2", "DevExpl", "N", "SumEvents")
	# add est. d.f. and p-value for spline variables
	if (numSplineVars == 1) {
		outData<-rbind(cfs, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
		rownames(outData) <- c(rownames(cfs), modelNB$smooth[[1]]$label, diagnosticVars)
		outData[modelNB$smooth[[1]]$label, 1] <- sumGAM$edf[1]
		outData[modelNB$smooth[[1]]$label, 4] <- sumGAM$s.pv[1]
	} else if (numSplineVars == 2) {
		outData<-rbind(cfs, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
		rownames(outData) <- c(rownames(cfs), modelNB$smooth[[1]]$label, modelNB$smooth[[2]]$label, diagnosticVars)
		outData[modelNB$smooth[[1]]$label, 1] <- sumGAM$edf[1]
		outData[modelNB$smooth[[2]]$label, 1] <- sumGAM$edf[2]
		outData[modelNB$smooth[[1]]$label, 4] <- sumGAM$s.pv[1]
		outData[modelNB$smooth[[2]]$label, 4] <- sumGAM$s.pv[2]
	} else {
		print("Invalid number of spline variables!")
	}
	# add the 'GCV'/'UMBRE' score
	if (sumGAM$method != "UBRE")
		print(paste("WARNING, sp.criterion method = ", sumGAM$method))
	outData["UBRE", 1] <- sumGAM$sp.criterion
	if (writeTheta == TRUE)
		outData["theta", 1] <- modelNB$family$getTheta()
	outData["logLik", 1] <- logLik(modelNB)
	outData["AIC", 1] <- AIC(modelNB)
	outData["NagelkerkeR2", 1] <- NagelkerkeR2(modelNB)$R2
	outData["AUC", 1] <-calcAUC(modelNB)
	outData["AdjR2", 1] <-sumGAM$r.sq	# appears to be a poor metric since it is often negative
	outData["DevExpl", 1] <-sumGAM$dev.expl
	outData["N", 1] <- length(modelNB$y)
	outData["SumEvents", 1] <- sum(modelNB$y)
	write.csv(outData, file=paste(outDir, fname, sep=""))
}

#### function to write pdf & jpg of GAM model (single variable)
#	requires valid outDir
gamPlotWithDensity <- function (gamModel, filePrefix, baseSmooth="", varNum=1, ylim=c(-2,1), xlim=c(-5,5), titleSuffix=NULL, densScale=1, dpi=300, convertToJpg=FALSE, vcovCL=NULL) {
#	gamModel <- gamBaseCntryFE; varNum <- 1; filePrefix <- "test"
#	ylim <- c(-2, 1); xlim <- c(-5, 5)
#	ylim <- c(-2.5, 1); xlim <- c(0, 100); densScale <- 10
#	browser()
	fsize <- 1.3	# 1.3 for 6x7 pdf
	filePrefix <- paste(filePrefix, varNum, baseSmooth, sep="")
	varName <- gamModel$smooth[[varNum]]$vn
	varVals <- gamModel$model[[varName]]
	pdf(file=paste(outDir, filePrefix, ".pdf", sep=""), width=6, height=7)	# too hard to re-size much
	plot.new()
	leftMarg <- 4; rightMarg <- 1; splitPoint <- 0.25
	# fig=(x1, x2, y1, y2), margins (bottom, left, top, right)
	#	http://www.statmethods.net/advgraphs/layout.html
	#	http://research.stowers-institute.org/efg/R/Graphics/Basics/mar-oma/index.htm
	par(fig=c(0,1,splitPoint,1), oma=c(0,0,0,0), mar=c(0.5,leftMarg,2,rightMarg), new=TRUE)
	# use 'select' to access GAM variable
	# axes=F generates a warning, but it is OK; use x/yaxt="n" & bty="n" instead
	if (is.null(vcovCL))
		plot(gamModel, select=varNum, se=2, ylab="Coefficient Estimate", ylim=ylim, xlim=xlim, cex.lab=fsize, shade=T, rug=F, xaxt="n", bty="o") # 'main' title doesn't work here
	else
		myPlotGAM(gamModel, select=varNum, se=2, ylab="Coefficient Estimate", ylim=ylim, xlim=xlim, cex.lab=fsize, shade=T, rug=F, xaxt="n", bty="o", vcovCL=vcovCL)
	axis(1, lab=F)
	title(main=paste("Spline Estimates for ", varName, titleSuffix, sep=""), cex.main=fsize+0.2)
	legend("top", c("Coefficient Estimates", "95% Confidence Interval"), lty=1, lwd=c(1.5,10), col=c("black", "gray80"), cex=c(1.1))
#	legend("top", c("Coefficient Estimates", "95% Confidence Interval", paste(varName, "Density")), lty=1, lwd=c(1.5,10,10), col=c("black", "gray80", "gray30"), cex=c(1.1))
	minVal <- min(varVals)
	maxVal <- max(varVals)
	lines(x=c(minVal, maxVal), y=c(0,0), lty=2, lwd=1.5, col="red")
	varDensity <- density(varVals)
	#sum(varDensity$y)*diff(varDensity$x[1:2])	# area under density curve = 1	
#	varDensity$y <- varDensity$y * densScale + (ylim[1] - (ylim[2]-ylim[1])*0.025)
	varDensity$y <- varDensity$y[varDensity$x > minVal & varDensity$x < maxVal]
	varDensity$x <- varDensity$x[varDensity$x > minVal & varDensity$x < maxVal]
	par(fig=c(0,1,0,splitPoint), oma=c(0,0,0,0), mar=c(4,leftMarg,0.5,rightMarg), new=TRUE)
	ylimDens <- c(0, max(0.5, max(varDensity$y)))	# set minimum yMax = 0.5
	plot(varDensity, col="white", xlim=xlim, ylim=ylimDens, main="", xlab=varName, cex.lab=fsize, yaxt="n")
	polygon(varDensity, col="gray30", border="gray30")
	yAxisPos <- c(0.0, 0.5)
	axis(2, at=yAxisPos, cex.axis=fsize, lab=F)	# set up Y-axis
	text(par("usr")[1] - (xlim[2]-xlim[1])/18, yAxisPos, srt=90, labels=yAxisPos, xpd=TRUE)
	dev.off()
	if (convertToJpg == TRUE) {
		setwd(outDir)
		shell(paste("convert -trim -bordercolor White -border 20 -density ", dpi, " ", filePrefix, ".pdf ", filePrefix, ".jpg", sep=""))
	}
}
	
#### myGLM model function
#	funcForm defaults to "negbin", can also be "ols"
#	optionally write model to file; defaults to FALSE
#	default data frame = longData
#	fixed effects should be included in glmVars list
#		Fixed/random effects reference: http://dss.princeton.edu/training/Panel101R.pdf
myGLM <- function(depVar, glmVars, writeFile=FALSE, modelName="", dataFr=longData, funcForm="negbin") {
	startTime <- proc.time()
	glmText <- paste(depVar, " ~ ", paste(glmVars, "+ ", collapse=""), sep="")
	glmFormula <- as.formula(substr(glmText, 1, nchar(glmText)-2))
	
	if (funcForm == "ols")
		glmObj <- glm(glmFormula, data=dataFr, family=gaussian())
		# lm() yields same results as glm(family=gaussian())
		#glmObj <- lm(glmFormula, data=dataFr)
	else if (funcForm == "negbin")
		# add glm.control for UCDP events
		glmObj <- glm.nb(glmFormula, data=dataFr, control=glm.control(maxit=100))
	else
		print(paste("Error: Invalid funcForm, ", funcForm))
		
	if (writeFile) {
		filename <- paste(depVar, modelName, "GLM", funcForm, ".csv", sep="")
		if (funcForm == "negbin")
			writeGLM(glmObj, filename, clustErrors=TRUE)
		else
			writeGLM(glmObj, filename, clustErrors=TRUE, writeTheta=FALSE)
	}
	# GVIF^(1/(2*Df)) > 2 means more than 2x a decrease in the precision of estimation due to collinearity
	#	confidence intervals twice as wide as they would be for uncorrelated predictors
	if (any(is.na(coef(glmObj))))
		print("can't perform vif test due to model NAs")
	else
		print(vif(glmObj)[vif(glmObj)[,3] > 2, , drop=FALSE])	# use 'drop' to retain rowname
	endTime <- proc.time()
	print(paste("myGLM() execution time, model ", depVar, " ", modelName, ": ", round(endTime[3]-startTime[3]), " seconds", sep="")); flush.console()
	glmObj
}

#### myGAM model function
#	funcForm defaults to "negbin", can also be "logit" or "ols"
#	optionally write model to files; defaults to TRUE
#	default data frame = longData
#	fixed effects should be included in glmVars list
#	use gamma = 1.4 to avoid overfitting, see p256 of Wood 2006
#	grabs global 'baseSmooth' value (an oversight; TODO: extract from gamVars text)
myGAM <- function(depVar, gamVars, thetaNB, writeFile=TRUE, modelName="", dataFr=longData, funcForm="negbin", gammaVal = 1.4) {
	# modelName <- "Base"
	startTime <- proc.time()
	gamText <- paste(depVar, " ~ ", paste(gamVars, "+ ", collapse=""), sep="")
	gamFormula <- as.formula(substr(gamText, 1, nchar(gamText)-2))
	
	if (funcForm == "logit")
		gamObj <- gam(gamFormula, data=dataFr, family=binomial(link = "logit"), gamma=gammaVal)
	else if (funcForm == "ols")
		gamObj <- gam(gamFormula, data=dataFr, family=gaussian(link = "identity"), gamma=gammaVal)
	else if (funcForm == "negbin")
		gamObj <- gam(gamFormula, data=dataFr, family=negbin(theta=thetaNB), gamma=gammaVal)
	else
		print(paste("Error: Invalid funcForm, ", funcForm))
		
	if (writeFile) {
		filename <- paste(depVar, modelName, "GAM", funcForm, sep="")
		if (funcForm == "logit" | funcForm == "ols")
			writeGAM(gamObj, paste(filename,baseSmooth,".csv", sep=""), numSplineVars=2, writeTheta=FALSE, clustErrors=TRUE, dataFr=dataFr)
		else
			writeGAM(gamObj, paste(filename,baseSmooth,".csv", sep=""), numSplineVars=2, clustErrors=TRUE, dataFr=dataFr)
		gamPlotWithDensity(gamObj, filename, baseSmooth=baseSmooth, varNum=1, ylim=c(-2,1), xlim=c(-5,5), titleSuffix=paste(",", modelName))
		gamPlotWithDensity(gamObj, filename, baseSmooth=baseSmooth, varNum=2, ylim=c(-2,1), xlim=c(-4,4), titleSuffix=paste(",", modelName))
	}
	endTime <- proc.time()
	print(paste("myGAM() execution time, model ", depVar, " ", modelName, ": ", round(endTime[3]-startTime[3]), " seconds", sep="")); flush.console()
	gamObj
}

#### function to calculate the predictive power of a given GAM model
#		requires valid longData object
calcGAMPredPower <- function (depVar, gamVars, thetaNB) {
	numVars <- length(gamVars)
	nb_AUCs <- rep(NA, numVars)
	names(nb_AUCs) <- gamVars
	startTime <- proc.time()
	for (i in 1:numVars){
		currIndVars <- gamVars[-i]
		# use the same theta for all regressions
		currGAM <- myGAM(depVar, currIndVars, thetaNB, writeFile=FALSE, "PredPower")
		nb_AUCs[i] <- calcAUC(currGAM)
	}
	endTime <- proc.time()
	print(paste("calcGAMPredPower() execution time, model ", depVar, ": ", round((endTime[3]-startTime[3])/60), " minutes", sep=""))
	nb_AUCs
}

print("Loaded functions getPolySlot(), calcAUC()")
print("   writeGLM(), writeGAM(), gamPlotWithDensity(), myGLM(), myGAM(), calcGAMPredPower()")

